package com.gihon.component.config;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

import org.apache.ibatis.reflection.MetaObject;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.baomidou.mybatisplus.core.handlers.StrictFill;
import com.baomidou.mybatisplus.core.metadata.TableInfo;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.gihon.component.entity.BaseEntity;
import com.gihon.component.entity.CompanyBaseEntity;
import com.gihon.component.util.Snowflake;
import com.gihon.component.web.auth.AuthUtils;

/**
 * MyBatis Plus 元数据处理类 用于自动 注入 id, createTime, updateTime, createUser,
 * updateUser 等字段
 *
 * @author baihe
 */
public class MyMetaObjectHandler implements MetaObjectHandler {

	/**
	 * id类型判断符
	 */
	private final static String ID_TYPE = "java.lang.String";

	private final static String FIELD_ID = "id";

	private Snowflake snowflake;

	/**
	 * @param workerId     终端ID
	 * @param dataCenterId 数据中心ID
	 */
	public MyMetaObjectHandler(long workerId, long dataCenterId) {
		super();
		snowflake = new Snowflake(workerId, dataCenterId);
	}

	/**
	 * 注意：不支持 复合主键 自动注入！！
	 * <p>
	 * 所有的继承了BaseEntity的实体，在insert时， id： id为空时， 通过IdGenerate生成唯一ID， 不为空则使用传递进来的id
	 * createUser, updateUser: 自动赋予 当前线程上的登录人id和companyId createTime, updateTime:
	 * 自动赋予 服务器的当前时间
	 * <p>
	 * 未继承任何父类的实体，且主键标注了 @TableId(value = "xxx", type = IdType.INPUT) 自动注入 主键
	 * 主键的字段名称任意
	 *
	 * @param metaObject
	 */
	@Override
	public void insertFill(MetaObject metaObject) {
		// 设置创建时间和创建人
		if (metaObject.getOriginalObject() instanceof BaseEntity) {
			BaseEntity entity = (BaseEntity) metaObject.getOriginalObject();
			if (entity.getCreateTime() == null) {
				entity.setCreateTime(LocalDateTime.now());
			}
			if (entity.getUpdateTime() == null) {
				entity.setUpdateTime(LocalDateTime.now());
			}
			if (entity.getCreateUser() == null) {
				entity.setCreateUser(AuthUtils.getUserId());
			}
			if (entity.getUpdateUser() == null) {
				entity.setUpdateUser(AuthUtils.getUserId());// 线程获取不到用户的时候
			}
			entity.setState(0);
			if (metaObject.getOriginalObject() instanceof CompanyBaseEntity) {
				CompanyBaseEntity entity2 = (CompanyBaseEntity) metaObject.getOriginalObject();

				if (entity2.getCompanyId() == null) {
					entity2.setCompanyId(AuthUtils.getCompanyId());// 线程获取不到用户的时候
				}
			}
			// 都是自增长
			return;
		}
		Long id = snowflake.nextId();
		if (metaObject.hasGetter(FIELD_ID)) {
			Object idVal = ID_TYPE.equals(metaObject.getGetterType(FIELD_ID).getName()) ? String.valueOf(id) : id;
			this.setFieldValByName(FIELD_ID, idVal, metaObject);
			return;
		}
//
		TableInfo tableInfo = TableInfoHelper.getTableInfo(metaObject.getOriginalObject().getClass());
		if (tableInfo == null) {
			return;
		}
		// id 字段名
		String keyProperty = tableInfo.getKeyProperty();
		if (keyProperty == null) {
			return;
		}
		// 反射得到 主键的值
		Object fieldValue = this.getFieldValByName(keyProperty, metaObject);
		// 判断ID 是否有值，有值就不
		if (!StringUtils.isEmpty(fieldValue)) {
			return;
		}
		Object idVal = fieldValue.getClass().getName().equalsIgnoreCase(ID_TYPE) ? String.valueOf(id) : id;
		this.setFieldValByName(keyProperty, idVal, metaObject);
	}



	/**
	 * 所有的继承了 BaseEntity的实体，在update时， updateUser: 自动赋予 当前线程上的登录人id updateTime: 自动赋予
	 * 服务器的当前时间
	 *
	 * @param metaObject
	 */
	@Override
	public void updateFill(MetaObject metaObject) {
		List<StrictFill> list = new ArrayList<>();
		list.add(StrictFill.of("updateTime", LocalDateTime.class, LocalDateTime.now()));
		Long userId = AuthUtils.getUserId();
		if (userId != null) {
			list.add(StrictFill.of("updateUser", Long.class,  AuthUtils.getUserId()));
		}
		
		this.strictUpdateFill(TableInfoHelper.getTableInfo(metaObject.getOriginalObject().getClass()),metaObject,list);
//        if (metaObject.getOriginalObject() instanceof BaseEntity) {
//        	BaseEntity entity = (BaseEntity) metaObject.getOriginalObject();
//            update(metaObject, entity);
//        } else {
//            //updateById updateBatchById update(T entity, Wrapper<T> updateWrapper);
//            Object et = metaObject.getValue(Constants.ENTITY);
//            if (et != null && et instanceof BaseEntity) {
//            	BaseEntity entity = (BaseEntity) et;
//                update(metaObject, entity, Constants.ENTITY + ".");
//            }
//        }
	}

//	private void update(MetaObject metaObject, BaseEntity entity, String et) {
//		if (entity.getUpdateUser() == null || entity.getUpdateUser().intValue() == 0) {
//			entity.setUpdateUser(AuthUtils.getUserId());// 线程获取不到用户的时候
//		}
//		if (entity.getUpdateTime() == null) {
//			entity.setUpdateTime(LocalDateTime.now());
//		}
//	}
//
//	private void update(MetaObject metaObject, BaseEntity entity) {
//		update(metaObject, entity, "");
//	}
}
