package com.gihon.component.service.impl;

import java.util.List;

import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.entity.rbac.GihonButton;
import com.gihon.component.entity.rbac.GihonMenu;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.mapper.rabc.GihonButtonMapper;
import com.gihon.component.service.GihonButtonService;
import com.google.common.collect.Lists;

/**
 * 直接继承没有多实现处理,这样不能用事务注解
 * 
 * @author baihe
 *
 */
@Service
public class GihonButtonServiceImpl extends ServiceImpl<GihonButtonMapper, GihonButton> implements GihonButtonService {

    /**
     * 获取菜单下按钮权限
     */
    public List<GihonButton> getMenuButton(Long menuId) {
        return this.lambdaQuery().eq(GihonButton::getMenuId, menuId).orderByAsc(GihonButton::getOrderNum).list();
    }

    /**
     * 获取菜单列表下按钮权限
     */
    public List<GihonButton> getMenuButton(List<Long> menuIdList,boolean idOnly) {
        if (menuIdList.isEmpty()) {
            return Lists.newArrayList();
        }
        LambdaQueryChainWrapper<GihonButton> wrapper = this.lambdaQuery().in(GihonButton::getMenuId, menuIdList).orderByAsc(GihonButton::getOrderNum);
        if(idOnly) {
    		wrapper.select(GihonButton::getId);
    	}
        return wrapper.list();
    }

    /**
     * 新增按钮权限
     */
    public boolean saveButton(GihonButton button) {
        int count = this.lambdaQuery().eq(GihonButton::getMenuId, button.getMenuId()).and(p -> {
            p.eq(GihonButton::getButtonName, button.getButtonName()).or().eq(GihonButton::getButtonCode, button.getButtonCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("按钮编码或者名称重复");
        }
        return this.save(button);

    }

    /**
     * 更新按钮权限
     */
    public boolean updateButton(GihonButton button) {
        GihonButton buttonOld = this.getById(button.getId());
        if (buttonOld == null) {
            throw new BusinessException("按钮不存在");
        }
        int count = this.lambdaQuery().eq(GihonButton::getMenuId, buttonOld.getMenuId()).ne(GihonButton::getId, button.getId()).and(p -> {
            p.eq(GihonButton::getButtonName, button.getButtonName()).or().eq(GihonButton::getButtonCode, button.getButtonCode());
        }).count();
        if (count > 0) {
            throw new BusinessException("按钮编码或者名称重复");
        }
        button.setMenuId(buttonOld.getMenuId());

        return this.updateById(button);

    }
}
