package com.gihon.component.service.impl;

import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.gihon.component.entity.rbac.GihonCompany;
import com.gihon.component.exception.BusinessException;
import com.gihon.component.mapper.GihonCompanyMapper;
import com.gihon.component.service.GihonCompanyService;
import com.gihon.component.vo.CompanyListReq;

/**
 * 直接继承没有多实现处理
 * 
 * @author baihe
 *
 */
@Service
public class GihonCompanyServiceImpl extends ServiceImpl<GihonCompanyMapper, GihonCompany> implements GihonCompanyService {

    /**
     * 分页获取公司列表
     */
    public IPage<GihonCompany> getCompanylistPage(CompanyListReq companyListReq) {
        Page<GihonCompany> page = new Page<GihonCompany>(companyListReq.getPageIndex(), companyListReq.getPageSize());
        return this.lambdaQuery().like(StringUtils.hasText(companyListReq.getCompanyName()), GihonCompany::getCompanyName, companyListReq.getCompanyName())
                .page(page);
    }

    /**
     * 激活(禁用)公司
     */
    public boolean enableCompany(Long companyId, Long userId, boolean enableFlag) {

        return this.lambdaUpdate().eq(GihonCompany::getId, companyId).eq(GihonCompany::getEnable, !enableFlag).set(GihonCompany::getEnable, enableFlag)
                .update();

    }

    /**
     * 新增公司
     */
    public boolean saveCompany(GihonCompany compnay) {
        int count = this.lambdaQuery().eq(GihonCompany::getCompanyName, compnay.getCompanyName()).count();
        if (count > 0) {
            throw new BusinessException("公司名称重复");
        }
        return this.save(compnay);

    }

    /**
     * 更新公司
     */
    public boolean updateCompany(GihonCompany company) {
        GihonCompany compnayOld = this.getById(company.getId());
        if (compnayOld == null) {
            throw new BusinessException("公司不存在");
        }
        int count = this.lambdaQuery().eq(GihonCompany::getCompanyName, company.getCompanyName()).ne(GihonCompany::getId, company.getId()).count();
        if (count > 0) {
            throw new BusinessException("公司名称重复");
        }
        GihonCompany compnayUpdate = new GihonCompany();
        compnayUpdate.setId(company.getId());
        compnayUpdate.setCompanyDescription(company.getCompanyDescription());
        compnayUpdate.setCompanyName(company.getCompanyName());
        compnayUpdate.setTelephone(company.getTelephone());
        compnayUpdate.setAddress(company.getAddress());
        return this.updateById(compnayUpdate);

    }
}
