package com.gihon.component.response;

import static com.gihon.component.response.ResponseStatus.NO_DATA;
import static com.gihon.component.response.ResponseStatus.SUCCESS;

import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

/**
 * 普通的响应值封装
 * 
 * @author baihe
 *
 * @param <T>
 */
@Data
@ApiModel("响应实体")
public class Response<T> {

	@ApiModelProperty(value = "返回标志")
	private int status;
	@ApiModelProperty(value = "描述")
	private String message;
	@ApiModelProperty(value = "数据")
	private T data;

	protected Response() {
	}

	protected Response(int status) {
		this.status = status;
	}

	protected Response(int status, String message) {
		this.status = status;
		this.message = message;
	}

	protected Response(int status, T data) {
		this.status = status;
		this.data = data;
	}

	protected Response(int status, String message, T data) {
		this.status = status;
		this.data = data;
		this.message = message;
	}

	public static <T> Response<T> ok() {
		return ok(null);
	}

	public static <T> Response<T> ok(T data) {
		return new Response<T>(SUCCESS.getCode(), SUCCESS.getDesc(), data);
	}

	public static <T> Response<T> noData() {
		return new Response<T>(NO_DATA.getCode(), NO_DATA.getDesc(), null);
	}

	public static <T> Response<T> error(ResponseStatus status, String message) {
		return new Response<T>(status.getCode(), message, null);
	}

	public static <T> Response<T> error(ResponseStatus status) {
		return error(status, status.getDesc());
	}

	public static <T> Response<PageBean<T>> okPage(IPage<T> data) {
		PageBean<T> pageBean = new PageBean<T>(data);
		return new Response<PageBean<T>>(SUCCESS.getCode(), SUCCESS.getDesc(), pageBean);
	}
}
