package com.gihon.component.rbac.service.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.gihon.component.enums.ModuleType;
import com.gihon.component.properties.GihonCommonProperties;
import com.gihon.component.properties.RedisConstants;
import com.gihon.component.rbac.service.PermissionService;
import com.gihon.component.util.JacksonJsonUtils;
import com.gihon.component.web.auth.AuthConstans;
import com.gihon.component.web.auth.AuthUser;
import com.gihon.component.web.auth.AuthUtils;
import com.gihon.component.web.vo.MenuVO;
import com.gihon.component.web.vo.PermissionVO;

@Service
public class PermissionServiceImpl implements PermissionService {

    @Autowired
    private StringRedisTemplate stringRedisTemplate;

    @Autowired
    private GihonCommonProperties gihonCommonProperties;
    
    @Override
    public List<MenuVO> getPermission() {
        
        String menuJson = (String) stringRedisTemplate.opsForHash().get(RedisConstants.MENU_TREE_REDIS,
                gihonCommonProperties.getModuleCode() + RedisConstants.SEP + gihonCommonProperties.getModuleType());
        TypeReference<List<MenuVO>> valueTypeRef = new TypeReference<List<MenuVO>>() {
        };
        List<MenuVO> root = JacksonJsonUtils.readObject(menuJson, valueTypeRef);
        //原始
        List<MenuVO> list = new ArrayList<>(root.size());
        
        AuthUser user = AuthUtils.getUser();
        if (AuthConstans.SUPER_ADMIN.equals(user.getUsername()) && gihonCommonProperties.getModuleType() == ModuleType.WEB.getCode()) {
           
            return root;
        }
        
        // 获取全部的角色
        List<String> role = user.getRoleList();
        //返回有权限的终极菜单。其他不需要后台控制的菜单有前端自行展示
        cloneCopyMenu(root,list,role);
        
        
        return list;
    }

    /**
     * 递归查询菜单Copy
     * 
     * @param vo
     * @param list
     */
    private void cloneCopyMenu(List<MenuVO> root, List<MenuVO> list, List<String> role) {
        if(role==null ||role.isEmpty()) {
            //无角色就无权限
            return;
        }
        for (MenuVO voi : root) {
            List<PermissionVO> permissions = voi.getPermissions();
            if (permissions == null || permissions.isEmpty()) {
                continue;
            }
            List<PermissionVO> pt = permissions.stream().filter(poi -> {
                if (poi.getRoleList() == null || poi.getRoleList().isEmpty()) {//未配置角色就是只有超级管理员有权限@See AuthServiceImpl#checkPermission
                    return false;
                }else if(poi.getRoleList().stream().anyMatch(s->role.contains(s))) {//角色是任意一个即可
                    return true;
                }
                return false;
            }).collect(Collectors.toList());
            
            if(pt.isEmpty()) {//菜单需要有任意一个操作权限
                continue;
            }
            MenuVO mv = new MenuVO();
            mv.setMenuCode(voi.getMenuCode());
            mv.setMenuId(voi.getMenuId());
            mv.setPermissions(pt);
            list.add(mv);
        }
    }
    
}
