package com.gihon.component.web.security;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.springframework.beans.factory.InitializingBean;
import org.springframework.security.access.ConfigAttribute;
import org.springframework.security.access.SecurityConfig;
import org.springframework.security.web.FilterInvocation;
import org.springframework.security.web.access.intercept.FilterInvocationSecurityMetadataSource;
import org.springframework.security.web.util.matcher.AntPathRequestMatcher;
import org.springframework.security.web.util.matcher.RequestMatcher;

public class GihonInvocationSecurityMetadataSource implements InitializingBean, FilterInvocationSecurityMetadataSource {

    private GihonSecurityService securityService;
    private HashMap<RequestMatcher, Collection<ConfigAttribute>> requestMap = new HashMap<RequestMatcher, Collection<ConfigAttribute>>();
    private Collection<ConfigAttribute> allAttribute = new HashSet<ConfigAttribute>();

    /**
     * 初始化所有资源和资源对应的角色
     */
    public void initResources() {
        if (null == securityService) {
            throw new RuntimeException(GihonSecurityService.class.getName() + "接口的实例为空，无法初始化资源");
        } else {
            // 读取所有角色
            LinkedList<String> allRoleNameList = securityService.getAllRoles();
            SecurityConfig attrConfig = null;
            if (null != allRoleNameList && allRoleNameList.size() > 0) {
                for (String oneOfAllRoleNames : allRoleNameList) {
                    if (null != oneOfAllRoleNames && !"".equals(oneOfAllRoleNames)) {
                        attrConfig = new SecurityConfig(oneOfAllRoleNames);
                        allAttribute.add(attrConfig);
                    }
                }
            }
            // 查询每个资源所对应的所有角色 按照url排序
            LinkedList<String[]> resourcesRoleRelationList = securityService.getAllResourceAndRoles();
            Collection<ConfigAttribute> array = new ArrayList<ConfigAttribute>();
            RequestMatcher matcher = null;
            SecurityConfig securityConfig = null;
            String tempUrl = "";
            String url = "";
            String roleName = "";
            if (null != resourcesRoleRelationList && resourcesRoleRelationList.size() > 0) {
                for (String[] resourcesRolesArr : resourcesRoleRelationList) {
                    url = resourcesRolesArr[0];
                    roleName = resourcesRolesArr[1];
                    if (!tempUrl.equals(url)) {
                        tempUrl = url;
                        matcher = new AntPathRequestMatcher(url);
                        array = new ArrayList<ConfigAttribute>();
                        requestMap.put(matcher, array);
                    }
                    securityConfig = new SecurityConfig(roleName);
                    array.add(securityConfig);
                }
            }
        }
    }

    /**
     * 根据资源获取需要的权限名称
     * 
     * @param object
     * @return
     */
    @Override
    public Collection<ConfigAttribute> getAttributes(Object object) throws IllegalArgumentException {
        // 把对象转化为请求
        final HttpServletRequest request = ((FilterInvocation) object).getRequest();
        // 循环整个Map 看看有没有可以匹配的,如果有匹配的就立刻返回
        Collection<ConfigAttribute> attrHashSet = new HashSet<ConfigAttribute>();
        for (Map.Entry<RequestMatcher, Collection<ConfigAttribute>> entry : requestMap.entrySet()) {
            if (entry.getKey().matches(request)) {
                attrHashSet.addAll(entry.getValue());
            }
        }
        if (attrHashSet.size() > 0) {
            return new ArrayList<ConfigAttribute>(attrHashSet);
        }
        return Collections.emptyList();
    }

    /**
     * 获取所有权限
     * 
     * @return
     */
    @Override
    public Collection<ConfigAttribute> getAllConfigAttributes() {
        return this.allAttribute;
    }

    @Override
    public boolean supports(Class<?> clazz) {
        return true;
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        initResources();
    }
}
