package com.gihon.component.web.security;

import java.io.IOException;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.AuthenticationEntryPoint;
import org.springframework.security.web.authentication.preauth.PreAuthenticatedAuthenticationToken;
import org.springframework.web.filter.OncePerRequestFilter;

import com.gihon.component.web.auth.AuthUtils;

public class GihonSecurityFilter extends OncePerRequestFilter {

    @Autowired
    private AuthenticationEntryPoint authenticationEntryPoint = new GihonAuthenticationEntryPoint();

    @Autowired
    private AuthenticationManager authenticationManager;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain chain) throws ServletException, IOException {
        try {

            String tokenValue = AuthUtils.parseToken(request);
            if (tokenValue == null) {
                logger.debug("No token in request, will continue chain.");
            } else {
                PreAuthenticatedAuthenticationToken authentication = new PreAuthenticatedAuthenticationToken(tokenValue, "");
                authentication.setDetails(tokenValue);
                Authentication authResult = authenticationManager.authenticate(authentication);
                logger.debug("Authentication success: " + authResult);
                SecurityContextHolder.getContext().setAuthentication(authResult);
            }
        } catch (AuthenticationException failed) {
            SecurityContextHolder.clearContext();

            logger.debug("Authentication request failed: " + failed);

            authenticationEntryPoint.commence(request, response, new InsufficientAuthenticationException(failed.getMessage(), failed));

            return;
        }

        chain.doFilter(request, response);
    }

    
}
