package com.gihon.component.web.security;

import java.util.ArrayList;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.util.StringUtils;

import com.gihon.component.web.service.SecurityTokenService;
import com.gihon.component.web.vo.SecurityUserInfo;

public class GihonUserDetailService implements UserDetailsService {

    @Autowired
    private SecurityTokenService tokenService;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        ArrayList<GrantedAuthority> authorities = new ArrayList<GrantedAuthority>();

        SecurityUserInfo user = tokenService.getUserInfo(username);
        if (user == null) {
            throw new UsernameNotFoundException("用户信息不正确");
        }
        List<String> rolesList = new ArrayList<String>();// 可以扩展从数据库中读取验证
        // TODO 从redis缓存中读取
        if (null != rolesList && rolesList.size() > 0) {
            for (String privilege : rolesList) {
                if (!StringUtils.isEmpty(privilege)) {
                    authorities.add(new SimpleGrantedAuthority(privilege));
                }
            }
        }

        authorities.add(new SimpleGrantedAuthority(SecurityConstans.PRIV_LOGIN));// 不显示给终端的，每个人都有的权限
        authorities.addAll(user.getAuthorities());
        user.setAuthorities(authorities);

        return user;
    }
}
