package com.gihon.component.web.security;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;

import com.gihon.component.web.vo.UserInfo;

public class SecurityUtil {

    private static Logger log = LoggerFactory.getLogger(SecurityUtil.class);

    public static String getCurrentToken() {
        return SecurityContextHolder.getContext().getAuthentication().getPrincipal().toString();
    }

    /**
     * Discription: [获取当前登录用户名字]
     */
    public static String getCurrentUserName() {
        String currentUserName = "";
        UserInfo userDetails = getCurrentUser();
        if (null != userDetails) {
            currentUserName = userDetails.getUsername();
        }
        return currentUserName;
    }

    /**
     * Discription: [获取当前登录用户ID]
     */
    public static Long getCurrentUserId() {
        Long currentUserName = null;
        UserInfo userDetails = getCurrentUser();
        if (null != userDetails) {
            currentUserName = userDetails.getId();
        }
        return currentUserName;
    }

    /**
     * Discription: [获取当前登录用户信息]
     */
    public static UserInfo getCurrentUser() {
        UserInfo userDetails = null;
        try {
            Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
            GihonAuthentication userAuthentication = (GihonAuthentication) authentication;
            userDetails = (UserInfo) userAuthentication.getUserDeatail();
        } catch (Exception e) {
            log.debug(e.getMessage());
        }
        return userDetails;
    }

    /**
     * Discription: [获取当前登录用户具有的所有权限名称]
     */
    public static List<String> getCurrentUserRoleNames() {
        Logger log = LoggerFactory.getLogger(SecurityUtil.class);
        List<String> roleList = new ArrayList<String>();
        UserDetails userDetails = null;
        try {
            if (null != SecurityContextHolder.getContext().getAuthentication()) {
                Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
                GihonAuthentication userAuthentication = (GihonAuthentication) authentication;
                userDetails = userAuthentication.getUserDeatail();
                Collection<? extends GrantedAuthority> authList = userDetails.getAuthorities();
                for (GrantedAuthority authority : authList) {
                    // PRIV_LOGIN 为系统预留拦截登录使用，不作为展现菜单使用的条件,也不应使用
                    if (!SecurityConstans.PRIV_LOGIN.equals(authority.getAuthority())) {
                        roleList.add(authority.getAuthority());
                    }
                }
            }
        } catch (Exception e) {
            log.error(e.getMessage());
        }
        return roleList;
    }

}
