package com.gihon.component.web.vo;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import com.gihon.component.entity.rbac.GihonUser;

import lombok.Data;

/**
 * 获取用户信息
 * 
 * @author baihe
 *
 */
@Data
public class SecurityUserInfo extends UserInfo implements  UserDetails  {

    /**
     * 
     */
    private static final long serialVersionUID = -3292038128292709733L;
    /**
     * 用户具有的角色集合（数据库中没有与之对应的字段，用户登录后系统自动注入）
     */
    private Collection<? extends GrantedAuthority> authorities;

    public SecurityUserInfo() { 
        
    }

    public SecurityUserInfo(GihonUser user) {
        super(user);
    }
    public SecurityUserInfo(UserInfo user) {
        this.setId(user.getId());
        this.setUsername(user.getUsername());
        this.setNickName(user.getNickName());
        this.setPassword(user.getPassword());
        this.setUsername(user.getUsername());
        this.setCompanyId(user.getCompanyId());
        this.setCompanyAdmin(user.getCompanyAdmin());
    }

    @Override
    public List<String> getRoleList() {
        if (authorities == null) {
            return null;
        }
        return authorities.stream().map(g -> g.getAuthority()).collect(Collectors.toList());
    }
    @Override
    public void setRoleList(List<String> roleList) {
        if (roleList == null) {
            return;
        }
        authorities = roleList.stream().map(r -> {
            return new SimpleGrantedAuthority(r);
        }).collect(Collectors.toList());
    }
}
